/***********************************************************
Copyright 1990 by Digital Equipment Corporation, Maynard, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute these examples for any
purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies and that both that copyright
notice and this permission notice appear in supporting documentation,
and that the name of Digital not be used in advertising or publicity
pertaining to distribution of the software without specific, written
prior permission.

DIGITAL AND THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT
OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
OR PERFORMANCE OF THIS SOFTWARE.

******************************************************************/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include "Box.h"
#include "Label.h"
#include "Pushbutton.h"
#include "Menu.h"
#include "MenuItem.h"

void Goodbye(widget, clientData, callData)
    Widget widget;
    caddr_t clientData, callData;
{
    exit(0);
}

void Widen(widget, clientData, callData)
    Widget widget;
    caddr_t clientData, callData;
{
    Widget w = (Widget) clientData;
    Arg arg[25];
    int n;
    String label;
    char buffer[100];

    n = 0;
    XtSetArg(arg[n], XtNlabel, &label);	n++;

    XtGetValues(w, arg, n);
    strcpy(&buffer[1], label);
    buffer[0] = buffer[1];

    n = 0;
    XtSetArg(arg[n], XtNlabel, buffer);	n++;
    XtSetArg(arg[n], XtNwidth, 0);	n++;
    XtSetArg(arg[n], XtNheight, 0);	n++;

    XtSetValues(w, arg, n);
}

void Narrow(widget, clientData, callData)
    Widget widget;
    caddr_t clientData, callData;
{
    Widget w = (Widget) clientData;
    Arg arg[25];
    int n;
    String label;
    char buffer[100];

    n = 0;
    XtSetArg(arg[n], XtNlabel, &label);	n++;

    XtGetValues(w, arg, n);
    strcpy(buffer, &label[1]);

    n = 0;
    XtSetArg(arg[n], XtNlabel, buffer);	n++;
    XtSetArg(arg[n], XtNwidth, 0);	n++;
    XtSetArg(arg[n], XtNheight, 0);	n++;

    XtSetValues(w, arg, n);
}

void Shuffle(widget, clientData, callData)
    Widget widget;
    caddr_t clientData, callData;
{
    Widget w = (Widget) clientData;
    Arg arg[25];
    int n;

    n = 0;
    XtSetArg(arg[n], XtNinsertBefore, 1);	n++;
    XtSetValues(w, arg, n);
}

Widget menushell;

static void RepositionMenu(w, event, params, num_params)
    Widget w;
    XEvent *event;
    String *params;
    Cardinal *num_params;
{
    XButtonEvent *b = &event->xbutton;
    Dimension width, height, border;
    Arg arg[25];
    int n;

    if (event->xany.type != ButtonPress) return;

    XtRealizeWidget(menushell);

    n = 0;
    XtSetArg(arg[n], XtNwidth, &width);		n++;
    XtSetArg(arg[n], XtNheight, &height);	n++;
    XtSetArg(arg[n], XtNborderWidth, &border);	n++;
    XtGetValues(menushell, arg, n);

    n = 0;
    XtSetArg(arg[n], XtNx, b->x_root - width/2 - border);	n++;
    XtSetArg(arg[n], XtNy, b->y_root - height/2 - border);	n++;
    XtSetValues(menushell, arg, n);
}

XtActionsRec actions[] = {
    {"RepositionMenu", RepositionMenu}
};

static char *menuTranslations = 
    "<Btn1Down>		: RepositionMenu() MenuPopup(menuShell) unhighlight()";


int main(argc, argv)
    unsigned int argc;
    char **argv;
{
    Widget toplevel, label, pushbutton, box, menu, exititem, first, widen, narrow, shuffle;
    Arg arg[25];
    unsigned int n;
    XtAppContext app;

    toplevel = XtAppInitialize(&app, "Menu",
	    (XrmOptionDescList) NULL, 0, &argc, argv,
	    (String *) NULL, (ArgList) NULL, 0);

    XtAppAddActions(app, actions, XtNumber(actions));

    box = XtCreateManagedWidget("box", boxWidgetClass, toplevel,
	(Arg *) NULL, 0);

    n = 0;
    XtSetArg(arg[n], XtNx, 10);	n++;
    XtSetArg(arg[n], XtNy, 10);	n++;
    XtSetArg(arg[n], XtNlabel, "Goodbye, world");	n++;

    label = XtCreateManagedWidget("label", labelWidgetClass, box,
	arg, n);

    n = 0;
    XtSetArg(arg[n], XtNx, 10); n++;
    XtSetArg(arg[n], XtNy, 40); n++;
    XtSetArg(arg[n], XtNlabel, "Exit menu");	n++;

    pushbutton = XtCreateManagedWidget("pushbutton", pushbuttonWidgetClass,
	box, arg, n);

    XtOverrideTranslations(pushbutton,
	    XtParseTranslationTable(menuTranslations));

    n = 0;
    XtSetArg(arg[n], XtNallowShellResize, TRUE);	n++;

    menushell = XtCreatePopupShell("menuShell",
	    overrideShellWidgetClass, pushbutton, arg, n);

    menu = XtCreateManagedWidget("menu", menuWidgetClass,
	    menushell, NULL, 0);

    first = XtCreateManagedWidget("aaaaa", menuItemWidgetClass, menu, NULL, 0);
    widen = XtCreateManagedWidget("widen", menuItemWidgetClass, menu, NULL, 0);
    narrow = XtCreateManagedWidget("narrow", menuItemWidgetClass, menu, NULL, 0);
    shuffle = XtCreateManagedWidget("shuffle", menuItemWidgetClass, menu, NULL, 0);
    exititem = XtCreateManagedWidget("exit", menuItemWidgetClass, menu, NULL, 0);

    XtAddCallback(widen, XtNcallback, Widen, first);
    XtAddCallback(narrow, XtNcallback, Narrow, first);
    XtAddCallback(shuffle, XtNcallback, Shuffle, first);
    XtAddCallback(exititem, XtNcallback, Goodbye, NULL);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app);
}
